# -*- coding: UTF-8 -*-
"""Showcase Tests"""
import copy
import json
from os.path import join

import pytest
from hdx.utilities.dictandlist import merge_two_dictionaries
from hdx.utilities.loader import load_yaml

from hdx.data.hdxobject import HDXError
from hdx.data.showcase import Showcase
from hdx.hdx_configuration import Configuration
from .test_vocabulary import vocabulary_mockshow
from . import MockResponse

showcase_resultdict = {
    'relationships_as_object': [],
    'num_tags': 2,
    'id': '05e392bf-04e0-4ca6-848c-4e87bba10746',
    'metadata_created': '2017-07-03T07:50:49.474517',
    'metadata_modified': '2017-07-03T08:12:43.726624',
    'author': 'Dan Mihaila',
    'author_email': 'dan@gmail.com',
    'state': 'active',
    'creator_user_id': '060468e4-2f33-4488-8504-c4b10cc34821',
    'type': 'showcase',
    'resources': [],
    'tags': [
        {
            'vocabulary_id': '4381925f-0ae9-44a3-b30d-cae35598757b',
            'state': 'active',
            'display_name': 'economics',
            'id': 'e1ff4b78-efcf-40e7-888b-7342169068e1',
            'name': 'economics'
        },
        {
            'vocabulary_id': '4381925f-0ae9-44a3-b30d-cae35598757b',
            'state': 'active',
            'display_name': 'health',
            'id': '8a43b269-78cb-4a85-a4ce-7e3e928bc487',
            'name': 'health'
        }
    ],
    'groups': [],
    'relationships_as_subject': [],
    'total_res_downloads': 0,
    'num_datasets': 2,
    'showcase_notes_formatted': '<p>lalala</p>',
    'image_url': 'visual.png',
    'revision_id': 'cc64364b-ede2-400a-bb9f-8e585a4f6399',
    'notes': 'My Showcase',
    'url': 'http://visualisation/url/',
    'title': 'MyShowcase1',
    'image_display_url': 'http://myvisual/visual.png',
    'name': 'showcase-1'
}

datasetsdict = load_yaml(join('tests', 'fixtures', 'dataset_search_results.yml'))
allsearchdict = load_yaml(join('tests', 'fixtures', 'showcase_all_search_results.yml'))


def mockshow(url, datadict):
    if 'package_list' in url:
        result = json.dumps(datasetsdict['results'])
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_package_list"}' % result)
    if '_show' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not show", "__type": "TEST ERROR: Not Show Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_show"}')
    result = json.dumps(showcase_resultdict)
    if datadict['id'] == '05e392bf-04e0-4ca6-848c-4e87bba10746' or datadict['id'] == 'TEST1':
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_show"}' % result)
    if datadict['id'] == 'TEST2':
        return MockResponse(404,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_show"}')
    if datadict['id'] == 'TEST3':
        return MockResponse(200,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_show"}')
    return MockResponse(404,
                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_show"}')


def mockallsearch(url, datadict):
    if 'search' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not search", "__type": "TEST ERROR: Not Search Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}')
    if 'metadata_modified' in datadict['fq']:
        newsearchdict = copy.deepcopy(allsearchdict)
        newsearchdict['results'] = [newsearchdict['results'][0]]
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}' % json.dumps(
                                newsearchdict))
    if datadict['q'] == 'ACLED':
        newsearchdict = copy.deepcopy(allsearchdict)
        if datadict['rows'] == 11:
            newsearchdict['results'].append(newsearchdict['results'][0])
        elif datadict['rows'] == 6:
            if datadict['start'] == 2:
                newsearchdict['results'] = newsearchdict['results'][2:8]
        elif datadict['rows'] == 5:
            if datadict['start'] == 0:
                newsearchdict['count'] = 5
                newsearchdict['results'] = newsearchdict['results'][:5]
            elif datadict['start'] == 5:
                newsearchdict['count'] = 6  # return wrong count
                newsearchdict['results'] = newsearchdict['results'][:5]
            else:
                newsearchdict['count'] = 0
                newsearchdict['results'] = list()
        result = json.dumps(newsearchdict)
        return MockResponse(200, '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}' % result)
    if datadict['q'] == '*:*':
        newsearchdict = copy.deepcopy(allsearchdict)
        newsearchdict['results'].extend(copy.deepcopy(newsearchdict['results']))
        for i, x in enumerate(newsearchdict['results']):
            x['id'] = '%s%d' % (x['id'], i)
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}' % json.dumps(
                                newsearchdict))
    if datadict['q'] == '"':
        return MockResponse(404,
                            '{"success": false, "error": {"message": "Validation Error", "__type": "Validation Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}')
    if datadict['q'] == 'ajyhgr':
        return MockResponse(200,
                            '{"success": true, "result": {"count": 0, "results": []}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}')
    return MockResponse(404,
                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}')


class TestShowcase:
    showcase_data = {
        'title': 'MyShowcase1',
        'notes': 'My Showcase',
        'package_id': '6f36a41c-f126-4b18-aaaf-6c2ddfbc5d4d',
        'image_url': 'http://myvisual/visual.png',
        'name': 'showcase-1',
        'url': 'http://visualisation/url/',
        'tags': [{'name': 'economy'}, {'name': 'health'}],
        'dataset_ids': ['a89c6260-6392-416e-bcbc-eb2c5f1d7add']
    }
    association = None

    @pytest.fixture(scope='class')
    def static_yaml(self):
        return join('tests', 'fixtures', 'config', 'hdx_showcase_static.yml')

    @pytest.fixture(scope='class')
    def static_json(self):
        return join('tests', 'fixtures', 'config', 'hdx_showcase_static.json')

    @pytest.fixture(scope='function')
    def read(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                if 'association_delete' in url:
                    TestShowcase.association = 'delete'
                    return MockResponse(200,
                                        '{"success": true, "result": null, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_package_association_delete"}')
                elif 'association_create' in url:
                    TestShowcase.association = 'create'
                    result = json.dumps(datadict)
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_package_association_create"}' % result)
                return mockshow(url, datadict)

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_create(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                if 'vocabulary' in url:
                    return vocabulary_mockshow(url, datadict)
                if url.endswith('show') or 'list' in url:
                    return mockshow(url, datadict)
                if 'create' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not create", "__type": "TEST ERROR: Not Create Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_create"}')

                resultdictcopy = copy.deepcopy(showcase_resultdict)
                resultdictcopy['state'] = datadict['state']
                result = json.dumps(resultdictcopy)
                if datadict['title'] == 'MyShowcase1':
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_create"}' % result)
                if datadict['title'] == 'MyShowcase2':
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_create"}')
                if datadict['title'] == 'MyShowcase3':
                    return MockResponse(200,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_create"}')

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_create"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_update(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                if 'vocabulary' in url:
                    return vocabulary_mockshow(url, datadict)
                if url.endswith('show') or 'list' in url:
                    return mockshow(url, datadict)
                if 'update' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not update", "__type": "TEST ERROR: Not Update Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_update"}')
                resultdictcopy = copy.deepcopy(showcase_resultdict)
                merge_two_dictionaries(resultdictcopy, datadict)

                result = json.dumps(resultdictcopy)
                if datadict['title'] == 'MyShowcase1':
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_update"}' % result)
                if datadict['title'] == 'MyShowcase2':
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_update"}')
                if datadict['title'] == 'MyShowcase3':
                    return MockResponse(200,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_update"}')

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_update"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_delete(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                decodedata = data.decode('utf-8')
                datadict = json.loads(decodedata)
                if url.endswith('show') or 'list' in url:
                    return mockshow(url, datadict)
                if 'delete' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not delete", "__type": "TEST ERROR: Not Delete Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_delete"}')
                if datadict['id'] == '05e392bf-04e0-4ca6-848c-4e87bba10746':
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_delete"}' % decodedata)

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=ckanext_showcase_delete"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def allsearch(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                return mockallsearch(url, datadict)

        Configuration.read().remoteckan().session = MockSession()

    def test_read_from_hdx(self, configuration, read):
        showcase = Showcase.read_from_hdx('05e392bf-04e0-4ca6-848c-4e87bba10746')
        assert showcase['id'] == '05e392bf-04e0-4ca6-848c-4e87bba10746'
        assert showcase['title'] == 'MyShowcase1'
        showcase = Showcase.read_from_hdx('TEST2')
        assert showcase is None
        showcase = Showcase.read_from_hdx('TEST3')
        assert showcase is None

    def test_create_in_hdx(self, configuration, post_create):
        showcase = Showcase()
        with pytest.raises(HDXError):
            showcase.create_in_hdx()
        showcase['id'] = '05e392bf-04e0-4ca6-848c-4e87bba10746'
        showcase['title'] = 'LALA'
        with pytest.raises(HDXError):
            showcase.create_in_hdx()

        showcase_data = copy.deepcopy(TestShowcase.showcase_data)
        showcase = Showcase(showcase_data)
        showcase.create_in_hdx()
        assert showcase['id'] == '05e392bf-04e0-4ca6-848c-4e87bba10746'
        assert showcase['state'] == 'active'

        showcase_data['title'] = 'MyShowcase2'
        showcase = Showcase(showcase_data)
        with pytest.raises(HDXError):
            showcase.create_in_hdx()

        showcase_data['title'] = 'MyShowcase3'
        showcase = Showcase(showcase_data)
        with pytest.raises(HDXError):
            showcase.create_in_hdx()

    def test_update_in_hdx(self, configuration, post_update):
        showcase = Showcase()
        showcase['id'] = 'NOTEXIST'
        with pytest.raises(HDXError):
            showcase.update_in_hdx()
        showcase['title'] = 'LALA'
        with pytest.raises(HDXError):
            showcase.update_in_hdx()

        showcase = Showcase.read_from_hdx('05e392bf-04e0-4ca6-848c-4e87bba10746')
        assert showcase['id'] == '05e392bf-04e0-4ca6-848c-4e87bba10746'
        assert showcase['title'] == 'MyShowcase1'

        showcase['name'] = 'TEST1'
        showcase['notes'] = 'lalalala'
        showcase.update_in_hdx()
        assert showcase['name'] == 'TEST1'
        assert showcase['notes'] == 'lalalala'
        assert showcase['state'] == 'active'
        expected = copy.deepcopy(showcase_resultdict)
        expected['notes'] = 'lalalala'
        expected['name'] = 'TEST1'
        expected['tags'] = [{'name': 'economics', 'vocabulary_id': '4381925f-0ae9-44a3-b30d-cae35598757b'}, {'name': 'health', 'vocabulary_id': '4381925f-0ae9-44a3-b30d-cae35598757b'}]
        assert showcase.get_old_data_dict() == expected

        showcase['name'] = 'NOTEXIST'
        with pytest.raises(HDXError):
            showcase.update_in_hdx()

        del showcase['name']
        with pytest.raises(HDXError):
            showcase.update_in_hdx()

        showcase_data = copy.deepcopy(TestShowcase.showcase_data)
        showcase_data['title'] = 'MyShowcase1'
        showcase_data['name'] = 'TEST1'
        showcase = Showcase(showcase_data)
        showcase.create_in_hdx()
        assert showcase['name'] == 'TEST1'
        assert showcase['notes'] == 'My Showcase'
        assert showcase['state'] == 'active'

    def test_delete_from_hdx(self, configuration, post_delete):
        showcase = Showcase.read_from_hdx('05e392bf-04e0-4ca6-848c-4e87bba10746')
        showcase.delete_from_hdx()
        del showcase['id']
        with pytest.raises(HDXError):
            showcase.delete_from_hdx()

    def test_update_yaml(self, configuration, static_yaml):
        showcase_data = copy.deepcopy(TestShowcase.showcase_data)
        showcase = Showcase(showcase_data)
        assert showcase['title'] == 'MyShowcase1'
        assert showcase['name'] == 'showcase-1'
        showcase.update_from_yaml(static_yaml)
        assert showcase['title'] == 'MyShowcase1'
        assert showcase['name'] == 'my-showcase-1'

    def test_update_json(self, configuration, static_json):
        showcase_data = copy.deepcopy(TestShowcase.showcase_data)
        showcase = Showcase(showcase_data)
        assert showcase['title'] == 'MyShowcase1'
        assert showcase['name'] == 'showcase-1'
        showcase.update_from_json(static_json)
        assert showcase['title'] == 'MyShowcase1'
        assert showcase['name'] == 'new-showcase-1'

    def test_tags(self, configuration):
        showcase_data = copy.deepcopy(TestShowcase.showcase_data)
        showcase = Showcase(showcase_data)
        assert showcase.get_tags() == ['economy', 'health']
        showcase.add_tag('wash')
        assert showcase.get_tags() == ['economy', 'health', 'water sanitation and hygiene - wash']
        showcase.add_tags(['sanitation'])
        assert showcase.get_tags() == ['economy', 'health', 'water sanitation and hygiene - wash']
        result = showcase.remove_tag('water sanitation and hygiene - wash')
        assert result is True
        assert showcase.get_tags() == ['economy', 'health']
        showcase['tags'] = None
        result = showcase.remove_tag('wash')
        assert result is False

    def test_datasets(self, configuration, read):
        showcase = Showcase.read_from_hdx('05e392bf-04e0-4ca6-848c-4e87bba10746')
        datasets = showcase.get_datasets()
        assert len(datasets) == 10
        assert datasets[0].data == datasetsdict['results'][0]
        dict4 = copy.deepcopy(datasetsdict['results'][4])
        del dict4['resources']
        assert datasets[4].data == dict4
        TestShowcase.association = None
        showcase.remove_dataset(datasets[0])
        assert TestShowcase.association == 'delete'
        TestShowcase.association = None
        assert showcase.add_dataset('a2f32edd-bac2-4940-aa58-49e565041055') is True
        assert TestShowcase.association == 'create'
        TestShowcase.association = None
        assert showcase.add_datasets([{'id': 'a2f32edd-bac2-4940-aa58-49e565041055'}, {'id': '6a5aebc1-f5a9-4842-8183-b8118228e71e'}]) is False
        assert TestShowcase.association == 'create'
        TestShowcase.association = None
        assert showcase.add_dataset({'name': 'TEST1'}) is True
        assert TestShowcase.association == 'create'
        TestShowcase.association = None
        with pytest.raises(HDXError):
            showcase.add_dataset('123')
        with pytest.raises(HDXError):
            showcase.add_dataset(123)

    def test_search_in_hdx(self, configuration, allsearch):
        showcases = Showcase.search_in_hdx('ACLED')
        assert len(showcases) == 10
        showcases = Showcase.search_in_hdx('ACLED', offset=2, limit=6)
        assert len(showcases) == 6
        showcases = Showcase.search_in_hdx(fq='metadata_modified:[2018-01-01T00:00:00.000Z TO NOW]')
        assert len(showcases) == 1
        showcases = Showcase.search_in_hdx('ajyhgr')
        assert len(showcases) == 0
        with pytest.raises(HDXError):
            Showcase.search_in_hdx('"')
        with pytest.raises(HDXError):
            Showcase.search_in_hdx('ACLED', rows=11)
        with pytest.raises(HDXError):
            # Test returned row counts per page mismatch (wrong count of 6 purposely in mocksearch)
            Showcase.search_in_hdx('ACLED', page_size=5)

    def test_get_all_showcases(self, configuration, allsearch):
        showcases = Showcase.get_all_showcases()
        assert len(showcases) == 20



# -*- coding: UTF-8 -*-
"""Organization Tests"""
import copy
import json
from os.path import join

import pytest
from hdx.utilities.dictandlist import merge_two_dictionaries
from hdx.utilities.loader import load_yaml

from hdx.data.hdxobject import HDXError
from hdx.data.organization import Organization
from hdx.data.user import User
from hdx.hdx_configuration import Configuration
from . import MockResponse, organization_data, user_data
from .test_user import user_mockshow

resultdict = load_yaml(join('tests', 'fixtures', 'organization_show_results.yml'))

organization_list = ['acaps', 'accion-contra-el-hambre-acf-spain', 'acf-west-africa', 'acled', 'acted',
                     'action-contre-la-faim', 'adeso', 'afd', 'afdb', 'afghanistan-protection-cluster']
searchdict = load_yaml(join('tests', 'fixtures', 'dataset_search_results.yml'))

organization_autocomplete = [{'title': 'INNAGO', 'id': '5a63012e-6c41-420c-8c33-e84b277fdc90', 'name': 'innago'}]


def organization_mockshow(url, datadict):
    if 'show' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not show", "__type": "TEST ERROR: Not Show Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_show"}')
    result = json.dumps(resultdict)
    if datadict['id'] == 'b67e6c74-c185-4f43-b561-0e114a736f19' or datadict['id'] == 'TEST1':
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_show"}' % result)
    if datadict['id'] == 'TEST2':
        return MockResponse(404,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_show"}')
    if datadict['id'] == 'TEST3':
        return MockResponse(200,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_show"}')
    return MockResponse(404,
                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_show"}')


def mocklist(url):
    if 'list' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not all", "__type": "TEST ERROR: Not All Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_list"}')
    return MockResponse(200,
                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_list"}' % json.dumps(
                            organization_list))


def mockgetdatasets(url, datadict):
    if 'search' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not search", "__type": "TEST ERROR: Not Search Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}')
    if datadict['fq'] == 'organization:acled':
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_search"}' % json.dumps(
                                searchdict))


class TestOrganization:
    @pytest.fixture(scope='class')
    def static_yaml(self):
        return join('tests', 'fixtures', 'config', 'hdx_organization_static.yml')

    @pytest.fixture(scope='class')
    def static_json(self):
        return join('tests', 'fixtures', 'config', 'hdx_organization_static.json')

    @pytest.fixture(scope='function')
    def read(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                return organization_mockshow(url, datadict)

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_create(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                if 'show' in url:
                    return organization_mockshow(url, datadict)
                if 'create' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not create", "__type": "TEST ERROR: Not Create Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_create"}')

                resultdictcopy = copy.deepcopy(resultdict)
                resultdictcopy['state'] = datadict['state']
                result = json.dumps(resultdictcopy)
                if datadict['name'] == 'MyOrganization1':
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_create"}' % result)
                if datadict['name'] == 'MyOrganization2':
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_create"}')
                if datadict['name'] == 'MyOrganization3':
                    return MockResponse(200,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_create"}')

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_create"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_update(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                if 'show' in url:
                    return organization_mockshow(url, datadict)
                if 'update' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not update", "__type": "TEST ERROR: Not Update Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_update"}')
                resultdictcopy = copy.deepcopy(resultdict)
                merge_two_dictionaries(resultdictcopy, datadict)

                result = json.dumps(resultdictcopy)
                if datadict['name'] == 'MyOrganization1':
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_update"}' % result)
                if datadict['name'] == 'MyOrganization2':
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_update"}')
                if datadict['name'] == 'MyOrganization3':
                    return MockResponse(200,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_update"}')

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_update"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_delete(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                decodedata = data.decode('utf-8')
                datadict = json.loads(decodedata)
                if 'show' in url:
                    return organization_mockshow(url, datadict)
                if 'delete' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not delete", "__type": "TEST ERROR: Not Delete Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_delete"}')
                if datadict['id'] == 'b67e6c74-c185-4f43-b561-0e114a736f19':
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_delete"}' % decodedata)

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_delete"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_list(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                return mocklist(url)

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def user_read(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                return user_mockshow(url, datadict)

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def datasets_get(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                return mockgetdatasets(url, datadict)

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_autocomplete(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                decodedata = data.decode('utf-8')
                datadict = json.loads(decodedata)
                if 'autocomplete' not in url or 'innago' not in datadict['q']:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not autocomplete", "__type": "TEST ERROR: Not Autocomplete Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_autocomplete"}')
                result = json.dumps(organization_autocomplete)
                return MockResponse(200,
                                    '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=organization_autocomplete"}' % result)

        Configuration.read().remoteckan().session = MockSession()

    def test_read_from_hdx(self, configuration, read, mocksmtp):
        organization = Organization.read_from_hdx('b67e6c74-c185-4f43-b561-0e114a736f19')
        assert organization['id'] == 'b67e6c74-c185-4f43-b561-0e114a736f19'
        assert organization['name'] == 'acled'
        organization = Organization.read_from_hdx('TEST2')
        assert organization is None
        organization = Organization.read_from_hdx('TEST3')
        assert organization is None

    def test_create_in_hdx(self, configuration, post_create):
        organization = Organization()
        with pytest.raises(HDXError):
            organization.create_in_hdx()
        organization['id'] = 'b67e6c74-c185-4f43-b561-0e114a736f19'
        organization['name'] = 'LALA'
        with pytest.raises(HDXError):
            organization.create_in_hdx()

        org_data = copy.deepcopy(organization_data)
        organization = Organization(org_data)
        organization.create_in_hdx()
        assert organization['id'] == 'b67e6c74-c185-4f43-b561-0e114a736f19'
        assert organization['state'] == 'active'

        org_data['name'] = 'MyOrganization2'
        organization = Organization(org_data)
        with pytest.raises(HDXError):
            organization.create_in_hdx()

        org_data['name'] = 'MyOrganization3'
        organization = Organization(org_data)
        with pytest.raises(HDXError):
            organization.create_in_hdx()

    def test_update_in_hdx(self, configuration, post_update):
        organization = Organization()
        organization['id'] = 'NOTEXIST'
        with pytest.raises(HDXError):
            organization.update_in_hdx()
        organization['name'] = 'LALA'
        with pytest.raises(HDXError):
            organization.update_in_hdx()

        organization = Organization.read_from_hdx('b67e6c74-c185-4f43-b561-0e114a736f19')
        assert organization['id'] == 'b67e6c74-c185-4f43-b561-0e114a736f19'
        assert organization['name'] == 'acled'

        organization['description'] = 'Humanitarian work'
        organization['id'] = 'b67e6c74-c185-4f43-b561-0e114a736f19'
        organization['name'] = 'MyOrganization1'
        organization.update_in_hdx()
        assert organization['id'] == 'b67e6c74-c185-4f43-b561-0e114a736f19'
        assert organization['description'] == 'Humanitarian work'
        assert organization['state'] == 'active'

        organization['id'] = 'NOTEXIST'
        with pytest.raises(HDXError):
            organization.update_in_hdx()

        del organization['id']
        with pytest.raises(HDXError):
            organization.update_in_hdx()

        org_data = copy.deepcopy(organization_data)
        org_data['name'] = 'MyOrganization1'
        org_data['id'] = 'b67e6c74-c185-4f43-b561-0e114a736f19'
        organization = Organization(org_data)
        organization.create_in_hdx()
        assert organization['id'] == 'b67e6c74-c185-4f43-b561-0e114a736f19'
        assert organization['name'] == 'MyOrganization1'
        assert organization['state'] == 'active'

    def test_delete_from_hdx(self, configuration, post_delete):
        organization = Organization.read_from_hdx('b67e6c74-c185-4f43-b561-0e114a736f19')
        organization.delete_from_hdx()
        del organization['id']
        with pytest.raises(HDXError):
            organization.delete_from_hdx()

    def test_update_yaml(self, configuration, static_yaml):
        org_data = copy.deepcopy(organization_data)
        organization = Organization(org_data)
        assert organization['name'] == 'MyOrganization1'
        assert organization['description'] == 'We do humanitarian work'
        organization.update_from_yaml(static_yaml)
        assert organization['name'] == 'MyOrg1'
        assert organization['description'] == 'Humanitarian organisation'

    def test_update_json(self, configuration, static_json):
        org_data = copy.deepcopy(organization_data)
        organization = Organization(org_data)
        assert organization['name'] == 'MyOrganization1'
        assert organization['description'] == 'We do humanitarian work'
        organization.update_from_json(static_json)
        assert organization['name'] == 'MyOrg1'
        assert organization['description'] == 'other'

    def test_get_all_organizations(self, configuration, post_list):
        organizations = Organization.get_all_organization_names()
        assert len(organizations) == 10

    def test_users(self, configuration, user_read):
        org_data = copy.deepcopy(resultdict)
        organization = Organization(org_data)
        organization['users'][0]['name'] = 'MyUser1'
        users = organization.get_users()
        assert len(users) == 1
        assert users[0]['name'] == 'MyUser1'
        organization.remove_user('9f3e9973-7dbe-4c65-8820-f48578e3ffea')
        users = organization.get_users()
        assert len(users) == 0
        userdata_copy = copy.deepcopy(user_data)
        userdata_copy['id'] = '9f3e9973-7dbe-4c65-8820-f48578e3ffea'
        user = User(userdata_copy)
        user['name'] = 'TEST1'
        user['capacity'] = 'member'
        del organization['users']
        organization.add_update_users([user])
        users = organization.get_users('member')
        assert len(users) == 1
        organization.remove_user(users[0])
        users = organization.get_users()
        assert len(users) == 0
        organization.add_update_users([user])
        users = organization.get_users('member')
        assert len(users) == 1
        organization.add_update_users(['MyUser1'], 'member')
        users = organization.get_users('member')
        assert len(users) == 2
        assert users[0]['name'] == 'MyUser1'
        organization.remove_user(users[0].data)
        users = organization.get_users('member')
        assert len(users) == 1
        organization.add_update_users([user], 'editor')
        organization.remove_user('')
        users = organization.get_users('editor')
        assert len(users) == 1
        organization.remove_user('NOT_EXIST')
        users = organization.get_users('editor')
        assert len(users) == 1
        users = organization.get_users('admin')
        assert len(users) == 0
        del organization['users'][0]['id']
        users = organization.get_users('member')
        assert len(users) == 1
        organization.add_update_user({'name': 'TEST1'}, 'member')
        users = organization.get_users('member')
        assert len(users) == 2
        with pytest.raises(HDXError):
            organization.add_update_users(123)
        with pytest.raises(HDXError):
            organization.add_update_user(123)
        with pytest.raises(HDXError):
            organization.remove_user(123)

    def test_get_datasets(self, configuration, datasets_get):
        org_data = copy.deepcopy(resultdict)
        organization = Organization(org_data)
        datasets = organization.get_datasets()
        assert len(datasets) == 10

    def test_autocomplete(self, configuration, post_autocomplete):
        assert Organization.autocomplete('innago') == organization_autocomplete



# -*- coding: UTF-8 -*-
"""Simple Facade Tests"""
from os.path import join

from hdx.utilities.useragent import UserAgentError, UserAgent

from hdx.facades import logging_kwargs
from hdx.version import get_api_version

logging_kwargs.update({
    'smtp_config_yaml': join('tests', 'fixtures', 'config', 'smtp_config.yml'),
})

import pytest

from . import testresult, my_testfn, my_excfn, my_testuafn, my_testkeyfn
from hdx.facades.simple import facade


class TestSimple:
    def test_facade(self, monkeypatch, hdx_config_yaml, project_config_yaml):
        UserAgent.clear_global()
        my_user_agent = 'test'
        testresult.actual_result = None
        facade(my_testfn, user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == 'https://data.humdata.org'
        UserAgent.clear_global()
        version = get_api_version()
        testresult.actual_result = None
        facade(my_testuafn, user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == 'HDXPythonLibrary/%s-%s' % (version, my_user_agent)
        UserAgent.clear_global()
        testresult.actual_result = None
        my_user_agent = 'lala'
        monkeypatch.setenv('USER_AGENT', my_user_agent)
        facade(my_testuafn, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == 'HDXPythonLibrary/%s-%s' % (version, my_user_agent)
        UserAgent.clear_global()
        testresult.actual_result = None
        facade(my_testuafn, user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == 'HDXPythonLibrary/%s-%s' % (version, my_user_agent)
        UserAgent.clear_global()
        testresult.actual_result = None
        my_preprefix = 'haha'
        monkeypatch.setenv('PREPREFIX', my_preprefix)
        facade(my_testuafn, user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == '%s:HDXPythonLibrary/%s-%s' % (my_preprefix, version, my_user_agent)
        UserAgent.clear_global()
        testresult.actual_result = None
        my_test_key = '1234'
        facade(my_testkeyfn, hdx_key=my_test_key, user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == my_test_key
        UserAgent.clear_global()
        testresult.actual_result = None
        monkeypatch.setenv('HDX_KEY', my_test_key)
        facade(my_testkeyfn, hdx_key='aaaa', user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == my_test_key
        UserAgent.clear_global()
        testresult.actual_result = None
        my_test_hdxsite = 'stage'
        facade(my_testfn, hdx_site=my_test_hdxsite, user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == 'https://%s.data-humdata-org.ahconu.org' % my_test_hdxsite
        UserAgent.clear_global()
        testresult.actual_result = None
        monkeypatch.setenv('HDX_SITE', my_test_hdxsite)
        facade(my_testfn, hdx_site='feature', user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == 'https://%s.data-humdata-org.ahconu.org' % my_test_hdxsite
        UserAgent.clear_global()
        my_test_hdxurl = 'http://other-data.humdata.org'
        monkeypatch.setenv('HDX_URL', my_test_hdxurl)
        facade(my_testfn, hdx_site='feature', user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == my_test_hdxurl
        UserAgent.clear_global()
        my_test_hdxurl2 = 'http://other-data.humdata.org/'
        monkeypatch.setenv('HDX_URL', my_test_hdxurl2)
        facade(my_testfn, hdx_site='feature', user_agent=my_user_agent, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        assert testresult.actual_result == my_test_hdxurl
        UserAgent.clear_global()

    def test_exception(self, hdx_config_yaml, project_config_yaml):
        UserAgent.clear_global()
        testresult.actual_result = None
        with pytest.raises(ValueError):
            facade(my_excfn, user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        UserAgent.clear_global()
        with pytest.raises(UserAgentError):
            facade(my_testuafn, hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        UserAgent.clear_global()



# -*'coding: UTF-8 -*-
"""Configuration Tests"""
from os.path import join

import ckanapi
import pytest
from hdx.utilities.loader import LoadError
from hdx.utilities.useragent import UserAgentError

from hdx.hdx_configuration import Configuration, ConfigurationError
from hdx.version import get_api_version


class TestConfiguration:
    @pytest.fixture(scope='class')
    def hdx_base_config_yaml(self, configfolder):
        return join(configfolder, 'hdx_base_config.yml')

    @pytest.fixture(scope='class')
    def hdx_base_config_json(self, configfolder):
        return join(configfolder, 'hdx_base_config.json')

    @pytest.fixture(scope='class')
    def hdx_config_json(self, configfolder):
        return join(configfolder, 'hdx_config.json')

    @pytest.fixture(scope='class')
    def hdx_missing_site_config_json(self, configfolder):
        return join(configfolder, 'hdx_missing_site_config.json')

    @pytest.fixture(scope='class')
    def project_config_json(self, configfolder):
        return join(configfolder, 'project_configuration.json')

    @pytest.fixture(scope='class')
    def user_agent_config_yaml(self, configfolder):
        return join(configfolder, 'user_agent_config.yml')

    @pytest.fixture(scope='class')
    def user_agent_config2_yaml(self, configfolder):
        return join(configfolder, 'user_agent_config2.yml')

    @pytest.fixture(scope='class')
    def user_agent_config3_yaml(self, configfolder):
        return join(configfolder, 'user_agent_config3.yml')

    @pytest.fixture(scope='class')
    def empty_yaml(self, configfolder):
        return join(configfolder, 'empty.yml')

    @pytest.fixture(scope='class')
    def user_agent_config_wrong_yaml(self, configfolder):
        return join(configfolder, 'user_agent_config_wrong.yml')

    def test_init(self, hdx_config_json, hdx_config_yaml, hdx_base_config_yaml, hdx_base_config_json, hdx_missing_site_config_json, project_config_json, project_config_yaml):
        default_config_file = Configuration.default_hdx_config_yaml
        Configuration.default_hdx_config_yaml = 'NOT EXIST'
        with pytest.raises(UserAgentError):
            Configuration()
        Configuration.default_hdx_config_yaml = default_config_file

        Configuration.default_hdx_config_yaml = hdx_config_yaml
        assert Configuration(user_agent='test').get_api_key() == '12345'
        Configuration.default_hdx_config_yaml = default_config_file

        with pytest.raises(IOError):
            Configuration(hdx_config_yaml='NOT_EXIST', project_config_yaml=project_config_yaml)

        with pytest.raises(IOError):
            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_yaml='NOT_EXIST',
                          project_config_yaml=project_config_yaml)

        with pytest.raises(IOError):
            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_json='NOT_EXIST',
                          project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(hdx_config_dict={'a': 1}, hdx_config_yaml=hdx_base_config_yaml,
                          hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(hdx_config_dict={'a': 1}, hdx_config_json=hdx_config_json,
                          hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(hdx_config_json=hdx_config_json, hdx_config_yaml=hdx_config_yaml,
                          hdx_base_config_json = hdx_base_config_json, project_config_yaml = project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_dict={'a': 1},
                          hdx_base_config_yaml=hdx_base_config_yaml, project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_dict={'a': 1},
                          hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_json=hdx_base_config_json,
                          hdx_base_config_yaml=hdx_base_config_yaml, project_config_yaml=project_config_yaml)

        with pytest.raises(IOError):
            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_yaml='NOT_EXIST')

        with pytest.raises(IOError):
            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_json='NOT_EXIST')

        with pytest.raises(ConfigurationError):
            Configuration(user_agent='test', hdx_config_json=hdx_missing_site_config_json,
                          project_config_json=project_config_json, hdx_base_config_json=hdx_base_config_json)

        with pytest.raises(ConfigurationError):
            Configuration(user_agent='test', hdx_site='NOT_EXIST', hdx_config_yaml=hdx_config_yaml,
                          project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_json=project_config_json,
                          project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
                          project_config_yaml=project_config_yaml)

        with pytest.raises(ConfigurationError):
            Configuration(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
                          project_config_json=project_config_json)

    def test_hdx_configuration_dict(self, project_config_yaml, mocksmtp):
        Configuration._create(user_agent='test', hdx_config_dict={'hdx_site': 'prod', 'hdx_read_only': True, 'hdx_key': 'abcde'},
                              hdx_base_config_dict={
                                                 'hdx_prod_site': {
                                                     'url': 'https://data.humdata.org',
                                                 },
                                                 'XYZ': {'567': 987}
                                             },
                              project_config_yaml=project_config_yaml)
        expected_configuration = {
            'hdx_site': 'prod',
            'hdx_read_only': True,
            'hdx_key': 'abcde',
            'hdx_prod_site': {
                'url': 'https://data.humdata.org',
            },
            'XYZ': {'567': 987},
            'approved_tags_vocabulary': 'Topics',
            'tags_list_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Accepted_Tags.csv',
            'tags_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Tag_Mapping.csv',
            'formats_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/resource_formats.json'
        }

        configuration = Configuration.read()
        assert configuration == expected_configuration
        version = get_api_version()
        assert configuration.get_user_agent() == 'HDXPythonLibrary/%s-test' % version

        Configuration._create(
            hdx_config_dict={'hdx_site': 'prod', 'hdx_read_only': True, 'hdx_key': 'abcde', 'user_agent': 'test'},
            hdx_base_config_dict={
                'hdx_prod_site': {
                    'url': 'https://data.humdata.org',
                },
                'XYZ': {'567': 987}
            },
            project_config_yaml=project_config_yaml)
        configuration = Configuration.read()
        expected_configuration['user_agent'] = 'test'
        assert configuration == expected_configuration
        assert configuration.get_user_agent() == 'HDXPythonLibrary/%s-test' % version

        smtp_initargs = {
            'host': 'localhost',
            'port': 123,
            'local_hostname': 'mycomputer.fqdn.com',
            'timeout': 3,
            'source_address': ('machine', 456),
        }
        username = 'user'
        password = 'pass'
        email_config_dict = {
            'connection_type': 'ssl',
            'username': username,
            'password': password
        }
        email_config_dict.update(smtp_initargs)

        recipients = ['larry@gmail.com', 'moe@gmail.com', 'curly@gmail.com']
        subject = 'hello'
        body = 'hello there'
        sender = 'me@gmail.com'
        mail_options = ['a', 'b']
        rcpt_options = [1, 2]

        with pytest.raises(ConfigurationError):
            configuration.emailer()
        configuration.setup_emailer(email_config_dict=email_config_dict)
        email = configuration.emailer()
        email.send(recipients, subject, body, sender=sender, mail_options=mail_options,
                                     rcpt_options=rcpt_options)
        assert email.server.type == 'smtpssl'
        assert email.server.initargs == smtp_initargs
        assert email.server.username == username
        assert email.server.password == password
        assert email.server.sender == sender
        assert email.server.recipients == recipients
        assert email.server.msg == '''Content-Type: text/plain; charset="us-ascii"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Subject: hello
From: me@gmail.com
To: larry@gmail.com, moe@gmail.com, curly@gmail.com

hello there'''
        assert email.server.send_args == {'mail_options': ['a', 'b'], 'rcpt_options': [1, 2]}

    def test_hdx_configuration_json(self, hdx_config_json, hdx_base_config_json, project_config_yaml):
        Configuration._create(user_agent='test', hdx_config_json=hdx_config_json,
                              hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)
        expected_configuration = {
            'hdx_site': 'stage',
            'hdx_read_only': False,
            'hdx_key': '54321',
            'hdx_prod_site': {
                'url': 'https://data.humdata.org',
            },
            'hdx_stage_site': {
                'url': 'https://stage.data-humdata-org.ahconu.org',
                'username': 'tumteetum',
                'password': 'tumteetumteetum'
            },
            'dataset': {'required_fields': [
                'name',
                'dataset_date',
            ]},
            'resource': {'required_fields': ['name', 'description']},
            'showcase': {'required_fields': ['name']},
            'approved_tags_vocabulary': 'Topics',
            'tags_list_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Accepted_Tags.csv',
            'tags_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Tag_Mapping.csv',
            'formats_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/resource_formats.json'
        }
        assert Configuration.read() == expected_configuration

    def test_hdx_configuration_yaml(self, hdx_config_yaml, hdx_base_config_yaml, project_config_yaml):
        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml,
                              hdx_base_config_yaml=hdx_base_config_yaml, project_config_yaml=project_config_yaml)
        expected_configuration = {
            'hdx_site': 'prod',
            'hdx_read_only': False,
            'hdx_key': '12345',
            'hdx_prod_site': {
                'url': 'https://data.humdata.org',
            },
            'hdx_stage_site': {
                'url': 'https://stage.data-humdata-org.ahconu.org',
                'username': 'lala',
                'password': 'lalala'
            },
            'dataset': {'required_fields': [
                'name',
                'title',
                'dataset_date',
            ]},
            'resource': {'required_fields': ['package_id', 'name', 'description']},
            'showcase': {'required_fields': ['name', 'title']},
            'approved_tags_vocabulary': 'Topics',
            'tags_list_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Accepted_Tags.csv',
            'tags_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Tag_Mapping.csv',
            'formats_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/resource_formats.json'
        }
        assert Configuration.read() == expected_configuration

    def test_project_configuration_dict(self, hdx_config_yaml):
        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml)
        expected_configuration = {
            'hdx_site': 'prod',
            'hdx_read_only': False,
            'hdx_key': '12345',
            'hdx_prod_site': {
                'url': 'https://data.humdata.org',
            },
            'hdx_demo_site': {
                'url': 'https://demo.data-humdata-org.ahconu.org',
            },
            'hdx_stage_site': {
                'url': 'https://stage.data-humdata-org.ahconu.org',
            },
            'hdx_feature_site': {
                'url': 'https://feature.data-humdata-org.ahconu.org',
            },
            'hdx_dev_site': {
                'url': 'https://dev.data-humdata-org.ahconu.org',
            },
            'dataset': {'required_fields': [
                'name',
                'private',
                'title',
                'notes',
                'dataset_source',
                'owner_org',
                'maintainer',
                'dataset_date',
                'data_update_frequency',
                'groups',
                'license_id',
                'methodology',
                'tags'
            ]},
            'dataset-requestable': {'required_fields': [
                'name',
                'title',
                'notes',
                'dataset_source',
                'owner_org',
                'maintainer',
                'dataset_date',
                'data_update_frequency',
                'groups',
                'tags',
                'field_names',
                'file_types',
                'num_of_rows'
            ]},
            'resource': {'required_fields': [
                'package_id',
                'name',
                'format',
                'description',
                'url_type',
                'resource_type']},
            'showcase': {'required_fields': [
                'name',
                'title',
                'notes',
                'url',
                'image_url',
                'tags'
            ]},
            'user': {'required_fields': [
                'name',
                'email',
                'password',
                'fullname',
                'about',
            ], 'ignore_on_update': 'password'},
            'organization': {'required_fields': [
                'name',
                'title',
                'description',
            ]},
            'resource view': {'required_fields': [
                'resource_id',
                'title',
                'view_type',
            ]},
            'vocabulary': {'required_fields': [
                'name',
                'tags',
            ]},
            'approved_tags_vocabulary': 'Topics',
            'tags_list_url': 'https://docs.google.com/spreadsheets/d/e/2PACX-1vRjeajloIuQl8mfTSHU71ZgbHSgYYUgHrLqyjHSuQJ-zMqS3SVM9hJqMs72L-84LQ/pub?gid=1739051517&single=true&output=csv',
            'tags_mapping_url': 'https://docs.google.com/spreadsheets/d/e/2PACX-1vRjeajloIuQl8mfTSHU71ZgbHSgYYUgHrLqyjHSuQJ-zMqS3SVM9hJqMs72L-84LQ/pub?gid=334970416&single=true&output=csv',
            'formats_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-ckan/master/ckanext-hdx_package/ckanext/hdx_package/config/resource_formats.json'
        }
        assert Configuration.read() == expected_configuration
        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_dict={'abc': '123'})
        expected_configuration['abc'] = '123'
        assert Configuration.read() == expected_configuration

    def test_project_configuration_json(self, hdx_config_yaml, project_config_json):
        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_json=project_config_json)
        expected_configuration = {
            'hdx_site': 'prod',
            'hdx_read_only': False,
            'hdx_key': '12345',
            'hdx_prod_site': {
                'url': 'https://data.humdata.org',
            },
            'hdx_demo_site': {
                'url': 'https://demo.data-humdata-org.ahconu.org',
            },
            'hdx_stage_site': {
                'url': 'https://stage.data-humdata-org.ahconu.org',
            },
            'hdx_feature_site': {
                'url': 'https://feature.data-humdata-org.ahconu.org',
            },
            'hdx_dev_site': {
                'url': 'https://dev.data-humdata-org.ahconu.org',
            },
            'my_param': 'abc',
            'dataset': {'required_fields': [
                'name',
                'private',
                'title',
                'notes',
                'dataset_source',
                'owner_org',
                'maintainer',
                'dataset_date',
                'data_update_frequency',
                'groups',
                'license_id',
                'methodology',
                'tags'
            ]},
            'dataset-requestable': {'required_fields': [
                'name',
                'title',
                'notes',
                'dataset_source',
                'owner_org',
                'maintainer',
                'dataset_date',
                'data_update_frequency',
                'groups',
                'tags',
                'field_names',
                'file_types',
                'num_of_rows'
            ]},
            'resource': {'required_fields': [
                'package_id',
                'name',
                'format',
                'description',
                'url_type',
                'resource_type']},
            'showcase': {'required_fields': [
                'name',
                'title',
                'notes',
                'url',
                'image_url',
                'tags'
            ]},
            'user': {'required_fields': [
                'name',
                'email',
                'password',
                'fullname',
                'about',
            ], 'ignore_on_update': 'password'},
            'organization': {'required_fields': [
                'name',
                'title',
                'description',
            ]},
            'resource view': {'required_fields': [
                'resource_id',
                'title',
                'view_type',
            ]},
            'vocabulary': {'required_fields': [
                'name',
                'tags',
            ]},
            'approved_tags_vocabulary': 'Topics',
            'tags_list_url': 'https://docs.google.com/spreadsheets/d/e/2PACX-1vRjeajloIuQl8mfTSHU71ZgbHSgYYUgHrLqyjHSuQJ-zMqS3SVM9hJqMs72L-84LQ/pub?gid=1739051517&single=true&output=csv',
            'tags_mapping_url': 'https://docs.google.com/spreadsheets/d/e/2PACX-1vRjeajloIuQl8mfTSHU71ZgbHSgYYUgHrLqyjHSuQJ-zMqS3SVM9hJqMs72L-84LQ/pub?gid=334970416&single=true&output=csv',
            'formats_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-ckan/master/ckanext-hdx_package/ckanext/hdx_package/config/resource_formats.json'
        }
        assert Configuration.read() == expected_configuration

    def test_project_configuration_yaml(self, hdx_config_yaml, project_config_yaml):
        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
        expected_configuration = {
            'hdx_site': 'prod',
            'hdx_read_only': False,
            'hdx_key': '12345',
            'hdx_prod_site': {
                'url': 'https://data.humdata.org',
            },
            'hdx_demo_site': {
                'url': 'https://demo.data-humdata-org.ahconu.org',
            },
            'hdx_stage_site': {
                'url': 'https://stage.data-humdata-org.ahconu.org',
            },
            'hdx_feature_site': {
                'url': 'https://feature.data-humdata-org.ahconu.org',
            },
            'hdx_dev_site': {
                'url': 'https://dev.data-humdata-org.ahconu.org',
            },
            'dataset': {'required_fields': [
                'name',
                'private',
                'title',
                'notes',
                'dataset_source',
                'owner_org',
                'maintainer',
                'dataset_date',
                'data_update_frequency',
                'groups',
                'license_id',
                'methodology',
                'tags'
            ]},
            'dataset-requestable': {'required_fields': [
                'name',
                'title',
                'notes',
                'dataset_source',
                'owner_org',
                'maintainer',
                'dataset_date',
                'data_update_frequency',
                'groups',
                'tags',
                'field_names',
                'file_types',
                'num_of_rows'
            ]},
            'resource': {'required_fields': [
                'package_id',
                'name',
                'format',
                'description',
                'url_type',
                'resource_type']},
            'showcase': {'required_fields': [
                'name',
                'title',
                'notes',
                'url',
                'image_url',
                'tags'
            ]},
            'user': {'required_fields': [
                'name',
                'email',
                'password',
                'fullname',
                'about',
            ], 'ignore_on_update': 'password'},
            'organization': {'required_fields': [
                'name',
                'title',
                'description',
            ]},
            'resource view': {'required_fields': [
                'resource_id',
                'title',
                'view_type',
            ]},
            'vocabulary': {'required_fields': [
                'name',
                'tags',
            ]},
            'approved_tags_vocabulary': 'Topics',
            'tags_list_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Accepted_Tags.csv',
            'tags_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/Tag_Mapping.csv',
            'formats_mapping_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/resource_formats.json'
        }
        assert Configuration.read() == expected_configuration

    def test_get_hdx_key_site(self, hdx_config_yaml, project_config_yaml):
        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml,
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        actual_configuration = Configuration.read()
        assert actual_configuration.get_api_key() == '12345'
        assert actual_configuration.get_hdx_site_url() == 'https://data.humdata.org'
        assert actual_configuration._get_credentials() is None
        assert actual_configuration.get_dataset_url('mydataset') == 'https://data.humdata.org/dataset/mydataset'

    def test_set_hdx_key_value(self, project_config_yaml):
        Configuration._create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        configuration = Configuration.read()
        assert configuration.get_api_key() == 'TEST_HDX_KEY'
        configuration.set_api_key('NEW API KEY')
        assert configuration.get_api_key() == 'NEW API KEY'
        Configuration._create(user_agent='test', hdx_site='prod', hdx_read_only=True,
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        assert Configuration.read().get_api_key() is None
        configuration = Configuration.read()
        configuration.set_api_key('TEST API KEY')
        assert configuration.get_api_key() is None
        configuration.set_read_only(False)
        assert configuration.get_api_key() == 'TEST API KEY'
        configuration.set_read_only(True)
        assert configuration.get_api_key() is None
        configuration.set_api_key('NEW API KEY')
        configuration.set_read_only(False)
        assert configuration.get_api_key() == 'NEW API KEY'

    def test_create_set_configuration(self, project_config_yaml):
        Configuration._create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        with pytest.raises(ConfigurationError):
            Configuration.create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                 hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        configuration = Configuration(user_agent='test', hdx_site='stage', hdx_key='OTHER_TEST_HDX_KEY',
                                      hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        Configuration.setup(configuration)
        assert Configuration.read() == configuration
        Configuration.delete()
        with pytest.raises(ConfigurationError):
            Configuration.read()
        Configuration.create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
                             hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        assert Configuration.read().get_api_key() == 'TEST_HDX_KEY'

    def test_remoteckan_validlocations(self, project_config_yaml):
        Configuration._create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        remoteckan = ckanapi.RemoteCKAN('http://lalala', apikey='12345',
                                        user_agent='HDXPythonLibrary/1.0')
        Configuration.read().setup_remoteckan(remoteckan=remoteckan)
        assert Configuration.read().remoteckan() == remoteckan
        remoteckan = ckanapi.RemoteCKAN('http://hahaha', apikey='54321',
                                        user_agent='HDXPythonLibrary/0.5')
        Configuration._create(user_agent='test',
                              remoteckan=remoteckan,
                              hdx_site='prod', hdx_key='TEST_HDX_KEY',
                              hdx_base_config_dict={},
                              project_config_yaml=project_config_yaml)
        assert Configuration.read().remoteckan() == remoteckan
        Configuration.read()._remoteckan = None
        with pytest.raises(ConfigurationError):
            Configuration.read().remoteckan()
        Configuration.delete()
        with pytest.raises(ConfigurationError):
            Configuration.read().remoteckan()

    def test_user_agent(self, user_agent_config_yaml, user_agent_config2_yaml, user_agent_config3_yaml,
                        empty_yaml, user_agent_config_wrong_yaml, project_config_yaml):
        Configuration._create(user_agent_config_yaml=user_agent_config_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        version = get_api_version()
        assert Configuration.read().remoteckan().user_agent == 'lala:HDXPythonLibrary/%s-myua' % version
        Configuration._create(user_agent_config_yaml=user_agent_config2_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        assert Configuration.read().remoteckan().user_agent == 'HDXPythonLibrary/%s-myuseragent' % version
        Configuration._create(user_agent_config_yaml=user_agent_config3_yaml, user_agent_lookup='lookup',  hdx_site='prod',
                              hdx_key='TEST_HDX_KEY', hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        assert Configuration.read().remoteckan().user_agent == 'HDXPythonLibrary/%s-mylookupagent' % version
        Configuration._create(user_agent_config_yaml=user_agent_config3_yaml, user_agent_lookup='lookup2',  hdx_site='prod',
                              hdx_key='TEST_HDX_KEY', hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        assert Configuration.read().remoteckan().user_agent == 'HDXPythonLibrary/%s-mylookupagent2' % version
        Configuration._create(user_agent='my_ua', preprefix='papa', hdx_site='prod', hdx_key='TEST_HDX_KEY',
                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        assert Configuration.read().remoteckan().user_agent == 'papa:HDXPythonLibrary/%s-my_ua' % version
        with pytest.raises(UserAgentError):
            Configuration._create(user_agent_config_yaml=user_agent_config3_yaml, user_agent_lookup='fail',
                                  hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        with pytest.raises(LoadError):
            Configuration._create(user_agent_config_yaml=empty_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        with pytest.raises(UserAgentError):
            Configuration._create(user_agent_config_yaml=user_agent_config_wrong_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
        with pytest.raises(UserAgentError):
            Configuration._create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)




# -*- coding: utf-8 -*-
"""Organization class containing all logic for creating, checking, and updating organizations."""
import logging
from os.path import join
from typing import Optional, List, Dict, Union, Any

import hdx.data.dataset
import hdx.data.user
from hdx.data.hdxobject import HDXObject, HDXError
from hdx.hdx_configuration import Configuration

logger = logging.getLogger(__name__)


class Organization(HDXObject):
    """Organization class containing all logic for creating, checking, and updating organizations.

    Args:
        initial_data (Optional[Dict]): Initial organization metadata dictionary. Defaults to None.
        configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
    """

    def __init__(self, initial_data=None, configuration=None):
        # type: (Optional[Dict], Optional[Configuration]) -> None
        if not initial_data:
            initial_data = dict()
        super(Organization, self).__init__(initial_data, configuration=configuration)

    @staticmethod
    def actions():
        # type: () -> Dict[str, str]
        """Dictionary of actions that can be performed on object

        Returns:
            Dict[str, str]: Dictionary of actions that can be performed on object
        """
        return {
            'show': 'organization_show',
            'update': 'organization_update',
            'create': 'organization_create',
            'delete': 'organization_delete',
            'list': 'organization_list',
            'autocomplete': 'organization_autocomplete'
        }

    def update_from_yaml(self, path=join('config', 'hdx_organization_static.yml')):
        # type: (str) -> None
        """Update organization metadata with static metadata from YAML file

        Args:
            path (str): Path to YAML dataset metadata. Defaults to config/hdx_organization_static.yml.

        Returns:
            None
        """
        super(Organization, self).update_from_yaml(path)

    def update_from_json(self, path=join('config', 'hdx_organization_static.json')):
        # type: (str) -> None
        """Update organization metadata with static metadata from JSON file

        Args:
            path (str): Path to JSON dataset metadata. Defaults to config/hdx_organization_static.json.

        Returns:
            None
        """
        super(Organization, self).update_from_json(path)

    @classmethod
    def read_from_hdx(cls, identifier, configuration=None):
        # type: (str, Optional[Configuration]) -> Optional['Organization']
        """Reads the organization given by identifier from HDX and returns Organization object

        Args:
            identifier (str): Identifier of organization
            configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.

        Returns:
            Optional[Organization]: Organization object if successful read, None if not
        """
        return cls._read_from_hdx_class('organization', identifier, configuration)

    def check_required_fields(self, ignore_fields=list()):
        # type: (List[str]) -> None
        """Check that metadata for organization is complete. The parameter ignore_fields should
        be set if required to any fields that should be ignored for the particular operation.

        Args:
            ignore_fields (List[str]): Fields to ignore. Default is [].

        Returns:
            None
        """
        self._check_required_fields('organization', ignore_fields)

    def update_in_hdx(self, **kwargs):
        # type: (Any) -> None
        """Check if organization exists in HDX and if so, update organization

        Returns:
            None
        """
        self._update_in_hdx('organization', 'id', **kwargs)

    def create_in_hdx(self, **kwargs):
        # type: (Any) -> None
        """Check if organization exists in HDX and if so, update it, otherwise create organization

        Returns:
            None
        """
        self._create_in_hdx('organization', 'id', 'name', **kwargs)

    def delete_from_hdx(self):
        # type: () -> None
        """Deletes a organization from HDX.

        Returns:
            None
        """
        self._delete_from_hdx('organization', 'id')

    def get_users(self, capacity=None):
        # type: (Optional[str]) -> List[hdx.data.user.User]
        """Returns the organization's users.

        Args:
            capacity (Optional[str]): Filter by capacity eg. member, admin. Defaults to None.
        Returns:
            List[hdx.data.user.User]: Organization's users.
        """
        users = list()
        usersdicts = self.data.get('users')
        if usersdicts is not None:
            for userdata in usersdicts:
                if capacity is not None and userdata['capacity'] != capacity:
                    continue
                id = userdata.get('id')
                if id is None:
                    id = userdata['name']
                user = hdx.data.user.User.read_from_hdx(id, configuration=self.configuration)
                user['capacity'] = userdata['capacity']
                users.append(user)
        return users

    def add_update_user(self, user, capacity=None):
        # type: (Union[hdx.data.user.User,Dict,str],Optional[str]) -> None
        """Add new or update existing user in organization with new metadata. Capacity eg. member, admin
        must be supplied either within the User object or dictionary or using the capacity argument (which takes
        precedence).

        Args:
            user (Union[User,Dict,str]): Either a user id or user metadata either from a User object or a dictionary
            capacity (Optional[str]): Capacity of user eg. member, admin. Defaults to None.

        Returns:
            None

        """
        if isinstance(user, str):
            user = hdx.data.user.User.read_from_hdx(user, configuration=self.configuration)
        elif isinstance(user, dict):
            user = hdx.data.user.User(user, configuration=self.configuration)
        if isinstance(user, hdx.data.user.User):
            users = self.data.get('users')
            if users is None:
                users = list()
                self.data['users'] = users
            if capacity is not None:
                user['capacity'] = capacity
            self._addupdate_hdxobject(users, 'name', user)
            return
        raise HDXError('Type %s cannot be added as a user!' % type(user).__name__)

    def add_update_users(self, users, capacity=None):
        # type: (List[Union[hdx.data.user.User,Dict,str]],Optional[str]) -> None
        """Add new or update existing users in organization with new metadata. Capacity eg. member, admin
        must be supplied either within the User object or dictionary or using the capacity argument (which takes
        precedence).

        Args:
            users (List[Union[User,Dict,str]]): A list of either user ids or users metadata from User objects or dictionaries
            capacity (Optional[str]): Capacity of users eg. member, admin. Defaults to None.

        Returns:
            None
        """
        if not isinstance(users, list):
            raise HDXError('Users should be a list!')
        for user in users:
            self.add_update_user(user, capacity)

    def remove_user(self, user):
        # type: (Union[hdx.data.user.User,Dict,str]) -> bool
        """Remove a user from the organization

        Args:
            user (Union[User,Dict,str]): Either a user id or user metadata either from a User object or a dictionary

        Returns:
            bool: True if user removed or False if not
        """
        return self._remove_hdxobject(self.data.get('users'), user)

    def get_datasets(self, query='*:*', **kwargs):
        # type: (str, Any) -> List[hdx.data.dataset.Dataset]
        """Get list of datasets in organization

        Args:
            query (str): Restrict datasets returned to this query (in Solr format). Defaults to '*:*'.
            **kwargs: See below
            sort (string): Sorting of the search results. Defaults to 'relevance asc, metadata_modified desc'.
            rows (int): Number of matching rows to return. Defaults to all datasets (sys.maxsize).
            start (int): Offset in the complete result for where the set of returned datasets should begin
            facet (string): Whether to enable faceted results. Default to True.
            facet.mincount (int): Minimum counts for facet fields should be included in the results
            facet.limit (int): Maximum number of values the facet fields return (- = unlimited). Defaults to 50.
            facet.field (List[str]): Fields to facet upon. Default is empty.
            use_default_schema (bool): Use default package schema instead of custom schema. Defaults to False.

        Returns:
            List[Dataset]: List of datasets in organization
        """
        return hdx.data.dataset.Dataset.search_in_hdx(query=query,
                                                      configuration=self.configuration,
                                                      fq='organization:%s' % self.data['name'], **kwargs)

    @staticmethod
    def get_all_organization_names(configuration=None, **kwargs):
        # type: (Optional[Configuration], Any) -> List[str]
        """Get all organization names in HDX

        Args:
            configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
            **kwargs: See below
            sort (str): Sort the search results according to field name and sort-order. Allowed fields are ‘name’, ‘package_count’ and ‘title’. Defaults to 'name asc'.
            organizations (List[str]): List of names of the groups to return.
            all_fields (bool): Return group dictionaries instead of just names. Only core fields are returned - get some more using the include_* options. Defaults to False.
            include_extras (bool): If all_fields, include the group extra fields. Defaults to False.
            include_tags (bool): If all_fields, include the group tags. Defaults to False.
            include_groups: If all_fields, include the groups the groups are in. Defaults to False.
            include_users (bool): If all_fields, include the organization users. Defaults to False.

        Returns:
            List[str]: List of all organization names in HDX
        """
        organization = Organization(configuration=configuration)
        return organization._write_to_hdx('list', kwargs)

    @classmethod
    def autocomplete(cls, name, limit=20, configuration=None):
        # type: (str, int, Optional[Configuration]) -> List
        """Autocomplete an organization name and return matches

        Args:
            name (str): Name to autocomplete
            limit (int): Maximum number of matches to return
            configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.

        Returns:
            List: Autocomplete matches
        """
        return cls._autocomplete(name, limit, configuration)



# -*- coding: UTF-8 -*-
"""Global fixtures"""
import smtplib
from os.path import join

import pytest

from hdx.hdx_configuration import Configuration
from hdx.hdx_locations import Locations


@pytest.fixture(scope='session')
def fixturesfolder():
    return join('tests', 'fixtures')


@pytest.fixture(scope='session')
def configfolder(fixturesfolder):
        return join(fixturesfolder, 'config')


@pytest.fixture(scope='session')
def hdx_config_yaml():
    return join('tests', 'fixtures', 'config', 'hdx_config.yml')


@pytest.fixture(scope='session')
def project_config_yaml():
    return join('tests', 'fixtures', 'config', 'project_configuration.yml')


@pytest.fixture(scope='session')
def locations():
     Locations.set_validlocations([{'display_name': 'Aruba', 'id': 'abw', 'name': 'abw', 'title': 'Aruba'},
                                   {'display_name': 'Afghanistan', 'id': 'afg', 'name': 'afg', 'title': 'Afghanistan'},
                                   {'display_name': 'Angola', 'id': 'ago', 'name': 'ago', 'title': 'Angola'},
                                   {'display_name': 'Anguilla', 'id': 'aia', 'name': 'aia', 'title': 'Anguilla'},
                                   {'display_name': 'Albania', 'id': 'alb', 'name': 'alb', 'title': 'Albania'},
                                   {'display_name': 'Andorra', 'id': 'and', 'name': 'and', 'title': 'Andorra'},
                                   {'display_name': 'Netherlands Antilles', 'id': 'ant', 'name': 'ant', 'title': 'Netherlands Antilles'},
                                   {'display_name': 'United Arab Emirates', 'id': 'are', 'name': 'are', 'title': 'United Arab Emirates'},
                                   {'display_name': 'Argentina', 'id': 'arg', 'name': 'arg', 'title': 'Argentina'},
                                   {'display_name': 'Armenia', 'id': 'arm', 'name': 'arm', 'title': 'Armenia'},
                                   {'display_name': 'American Samoa', 'id': 'asm', 'name': 'asm', 'title': 'American Samoa'},
                                   {'display_name': 'Antartica', 'id': 'ata', 'name': 'ata', 'title': 'Antartica'},
                                   {'display_name': 'French Southern Territories', 'id': 'atf', 'name': 'atf', 'title': 'French Southern Territories'},
                                   {'display_name': 'Antigua And Barbuda', 'id': 'atg', 'name': 'atg', 'title': 'Antigua And Barbuda'},
                                   {'display_name': 'Australia', 'id': 'aus', 'name': 'aus', 'title': 'Australia'},
                                   {'display_name': 'Austria', 'id': 'aut', 'name': 'aut', 'title': 'Austria'},
                                   {'display_name': 'Azerbajan', 'id': 'aze', 'name': 'aze', 'title': 'Azerbajan'},
                                   {'display_name': 'Burundi', 'id': 'bdi', 'name': 'bdi', 'title': 'Burundi'},
                                   {'display_name': 'Belgium', 'id': 'bel', 'name': 'bel', 'title': 'Belgium'},
                                   {'display_name': 'Benin', 'id': 'ben', 'name': 'ben', 'title': 'Benin'},
                                   {'display_name': 'Burkina Faso', 'id': 'bfa', 'name': 'bfa', 'title': 'Burkina Faso'},
                                   {'display_name': 'Bangladesh', 'id': 'bgd', 'name': 'bgd', 'title': 'Bangladesh'},
                                   {'display_name': 'Bulgaria', 'id': 'bgr', 'name': 'bgr', 'title': 'Bulgaria'},
                                   {'display_name': 'Bahrain', 'id': 'bhr', 'name': 'bhr', 'title': 'Bahrain'},
                                   {'display_name': 'Bahamas', 'id': 'bhs', 'name': 'bhs', 'title': 'Bahamas'},
                                   {'display_name': 'Bosnia And Herzegowina', 'id': 'bih', 'name': 'bih', 'title': 'Bosnia And Herzegowina'},
                                   {'display_name': 'Belarus', 'id': 'blr', 'name': 'blr', 'title': 'Belarus'},
                                   {'display_name': 'Belize', 'id': 'blz', 'name': 'blz', 'title': 'Belize'},
                                   {'display_name': 'Bermuda', 'id': 'bmu', 'name': 'bmu', 'title': 'Bermuda'},
                                   {'display_name': 'Bolivia (Plurinational State of)', 'id': 'bol', 'name': 'bol', 'title': 'Bolivia (Plurinational State of)'},
                                   {'display_name': 'Brazil', 'id': 'bra', 'name': 'bra', 'title': 'Brazil'},
                                   {'display_name': 'Barbados', 'id': 'brb', 'name': 'brb', 'title': 'Barbados'},
                                   {'display_name': 'Brunei Darussalam', 'id': 'brn', 'name': 'brn', 'title': 'Brunei Darussalam'},
                                   {'display_name': 'Bhutan', 'id': 'btn', 'name': 'btn', 'title': 'Bhutan'},
                                   {'display_name': 'Bouvet Island', 'id': 'bvt', 'name': 'bvt', 'title': 'Bouvet Island'},
                                   {'display_name': 'Botswana', 'id': 'bwa', 'name': 'bwa', 'title': 'Botswana'},
                                   {'display_name': 'Central African Republic', 'id': 'caf', 'name': 'caf', 'title': 'Central African Republic'},
                                   {'display_name': 'Canada', 'id': 'can', 'name': 'can', 'title': 'Canada'},
                                   {'display_name': 'Cocos (Keeling) Islands', 'id': 'cck', 'name': 'cck', 'title': 'Cocos (Keeling) Islands'},
                                   {'display_name': 'Switzerland', 'id': 'che', 'name': 'che', 'title': 'Switzerland'},
                                   {'display_name': 'Chile', 'id': 'chl', 'name': 'chl', 'title': 'Chile'},
                                   {'display_name': 'China', 'id': 'chn', 'name': 'chn', 'title': 'China'},
                                   {'display_name': "Côte d'Ivoire", 'id': 'civ', 'name': 'civ', 'title': "Côte d'Ivoire"},
                                   {'display_name': 'Cameroon', 'id': 'cmr', 'name': 'cmr', 'title': 'Cameroon'},
                                   {'display_name': 'Democratic Republic of the Congo', 'id': 'cod', 'name': 'cod', 'title': 'Democratic Republic of the Congo'},
                                   {'display_name': 'Congo', 'id': 'cog', 'name': 'cog', 'title': 'Congo'},
                                   {'display_name': 'Cook Islands', 'id': 'cok', 'name': 'cok', 'title': 'Cook Islands'},
                                   {'display_name': 'Colombia', 'id': 'col', 'name': 'col', 'title': 'Colombia'},
                                   {'display_name': 'Comoros', 'id': 'com', 'name': 'com', 'title': 'Comoros'},
                                   {'display_name': 'Cape Verde', 'id': 'cpv', 'name': 'cpv', 'title': 'Cape Verde'},
                                   {'display_name': 'Costa Rica', 'id': 'cri', 'name': 'cri', 'title': 'Costa Rica'},
                                   {'display_name': 'Cuba', 'id': 'cub', 'name': 'cub', 'title': 'Cuba'},
                                   {'display_name': 'Christmas Island', 'id': 'cxr', 'name': 'cxr', 'title': 'Christmas Island'},
                                   {'display_name': 'Cayman Islands', 'id': 'cym', 'name': 'cym', 'title': 'Cayman Islands'},
                                   {'display_name': 'Cyprus', 'id': 'cyp', 'name': 'cyp', 'title': 'Cyprus'},
                                   {'display_name': 'Czech Republic', 'id': 'cze', 'name': 'cze', 'title': 'Czech Republic'},
                                   {'display_name': 'Germany', 'id': 'deu', 'name': 'deu', 'title': 'Germany'},
                                   {'display_name': 'Djibouti', 'id': 'dji', 'name': 'dji', 'title': 'Djibouti'},
                                   {'display_name': 'Dominica', 'id': 'dma', 'name': 'dma', 'title': 'Dominica'},
                                   {'display_name': 'Denmark', 'id': 'dnk', 'name': 'dnk', 'title': 'Denmark'},
                                   {'display_name': 'Dominican Republic', 'id': 'dom', 'name': 'dom', 'title': 'Dominican Republic'},
                                   {'display_name': 'Algeria', 'id': 'dza', 'name': 'dza', 'title': 'Algeria'},
                                   {'display_name': 'Ecuador', 'id': 'ecu', 'name': 'ecu', 'title': 'Ecuador'},
                                   {'display_name': 'Egypt', 'id': 'egy', 'name': 'egy', 'title': 'Egypt'},
                                   {'display_name': 'Eritrea', 'id': 'eri', 'name': 'eri', 'title': 'Eritrea'},
                                   {'display_name': 'Western Sahara', 'id': 'esh', 'name': 'esh', 'title': 'Western Sahara'},
                                   {'display_name': 'Spain', 'id': 'esp', 'name': 'esp', 'title': 'Spain'},
                                   {'display_name': 'Estonia', 'id': 'est', 'name': 'est', 'title': 'Estonia'},
                                   {'display_name': 'Ethiopia', 'id': 'eth', 'name': 'eth', 'title': 'Ethiopia'},
                                   {'display_name': 'Finland', 'id': 'fin', 'name': 'fin', 'title': 'Finland'},
                                   {'display_name': 'Fiji', 'id': 'fji', 'name': 'fji', 'title': 'Fiji'},
                                   {'display_name': 'Falkland Islands (Malvinas)', 'id': 'flk', 'name': 'flk', 'title': 'Falkland Islands (Malvinas)'},
                                   {'display_name': 'France', 'id': 'fra', 'name': 'fra', 'title': 'France'},
                                   {'display_name': 'Faroe Islands', 'id': 'fro', 'name': 'fro', 'title': 'Faroe Islands'},
                                   {'display_name': 'Micronesia, Federated States Of', 'id': 'fsm', 'name': 'fsm', 'title': 'Micronesia, Federated States Of'},
                                   {'display_name': 'France, Metropolitan', 'id': 'fxx', 'name': 'fxx', 'title': 'France, Metropolitan'},
                                   {'display_name': 'Gabon', 'id': 'gab', 'name': 'gab', 'title': 'Gabon'},
                                   {'display_name': 'United Kingdom', 'id': 'gbr', 'name': 'gbr', 'title': 'United Kingdom'},
                                   {'display_name': 'Georgia', 'id': 'geo', 'name': 'geo', 'title': 'Georgia'},
                                   {'display_name': 'Ghana', 'id': 'gha', 'name': 'gha', 'title': 'Ghana'},
                                   {'display_name': 'Gibraltar', 'id': 'gib', 'name': 'gib', 'title': 'Gibraltar'},
                                   {'display_name': 'Guinea', 'id': 'gin', 'name': 'gin', 'title': 'Guinea'},
                                   {'display_name': 'Guadeloupe', 'id': 'glp', 'name': 'glp', 'title': 'Guadeloupe'},
                                   {'display_name': 'Gambia', 'id': 'gmb', 'name': 'gmb', 'title': 'Gambia'},
                                   {'display_name': 'Guinea-Bissau', 'id': 'gnb', 'name': 'gnb', 'title': 'Guinea-Bissau'},
                                   {'display_name': 'Equatorial Guinea', 'id': 'gnq', 'name': 'gnq', 'title': 'Equatorial Guinea'},
                                   {'display_name': 'Greece', 'id': 'grc', 'name': 'grc', 'title': 'Greece'},
                                   {'display_name': 'Grenada', 'id': 'grd', 'name': 'grd', 'title': 'Grenada'},
                                   {'display_name': 'Greenland', 'id': 'grl', 'name': 'grl', 'title': 'Greenland'},
                                   {'display_name': 'Guatemala', 'id': 'gtm', 'name': 'gtm', 'title': 'Guatemala'},
                                   {'display_name': 'French Guiana', 'id': 'guf', 'name': 'guf', 'title': 'French Guiana'},
                                   {'display_name': 'Guam', 'id': 'gum', 'name': 'gum', 'title': 'Guam'},
                                   {'display_name': 'Guyana', 'id': 'guy', 'name': 'guy', 'title': 'Guyana'},
                                   {'display_name': 'Hong Kong', 'id': 'hkg', 'name': 'hkg', 'title': 'Hong Kong'},
                                   {'display_name': 'Heard And McDonald Islands', 'id': 'hmd', 'name': 'hmd', 'title': 'Heard And McDonald Islands'},
                                   {'display_name': 'Honduras', 'id': 'hnd', 'name': 'hnd', 'title': 'Honduras'},
                                   {'display_name': 'Croatia', 'id': 'hrv', 'name': 'hrv', 'title': 'Croatia'},
                                   {'display_name': 'Haiti', 'id': 'hti', 'name': 'hti', 'title': 'Haiti'},
                                   {'display_name': 'Hungary', 'id': 'hun', 'name': 'hun', 'title': 'Hungary'},
                                   {'display_name': 'Indonesia', 'id': 'idn', 'name': 'idn', 'title': 'Indonesia'},
                                   {'display_name': 'India', 'id': 'ind', 'name': 'ind', 'title': 'India'},
                                   {'display_name': 'British Indian Ocean Territory', 'id': 'iot', 'name': 'iot', 'title': 'British Indian Ocean Territory'},
                                   {'display_name': 'Ireland', 'id': 'irl', 'name': 'irl', 'title': 'Ireland'},
                                   {'display_name': 'Iran (Islamic Republic Of)', 'id': 'irn', 'name': 'irn', 'title': 'Iran (Islamic Republic Of)'},
                                   {'display_name': 'Iraq', 'id': 'irq', 'name': 'irq', 'title': 'Iraq'},
                                   {'display_name': 'Iceland', 'id': 'isl', 'name': 'isl', 'title': 'Iceland'},
                                   {'display_name': 'Israel', 'id': 'isr', 'name': 'isr', 'title': 'Israel'},
                                   {'display_name': 'Italy', 'id': 'ita', 'name': 'ita', 'title': 'Italy'},
                                   {'display_name': 'Jamaica', 'id': 'jam', 'name': 'jam', 'title': 'Jamaica'},
                                   {'display_name': 'Jordan', 'id': 'jor', 'name': 'jor', 'title': 'Jordan'},
                                   {'display_name': 'Japan', 'id': 'jpn', 'name': 'jpn', 'title': 'Japan'},
                                   {'display_name': 'Kazakhstan', 'id': 'kaz', 'name': 'kaz', 'title': 'Kazakhstan'},
                                   {'display_name': 'Kenya', 'id': 'ken', 'name': 'ken', 'title': 'Kenya'},
                                   {'display_name': 'Kyrgyzstan', 'id': 'kgz', 'name': 'kgz', 'title': 'Kyrgyzstan'},
                                   {'display_name': 'Cambodia', 'id': 'khm', 'name': 'khm', 'title': 'Cambodia'},
                                   {'display_name': 'Kiribati', 'id': 'kir', 'name': 'kir', 'title': 'Kiribati'},
                                   {'display_name': 'Saint Kitts And Nevis', 'id': 'kna', 'name': 'kna', 'title': 'Saint Kitts And Nevis'},
                                   {'display_name': 'Korea, Republic Of', 'id': 'kor', 'name': 'kor', 'title': 'Korea, Republic Of'},
                                   {'display_name': 'Kuwait', 'id': 'kwt', 'name': 'kwt', 'title': 'Kuwait'},
                                   {'display_name': "Lao People's Democratic Republic", 'id': 'lao', 'name': 'lao', 'title': "Lao People's Democratic Republic"},
                                   {'display_name': 'Lebanon', 'id': 'lbn', 'name': 'lbn', 'title': 'Lebanon'},
                                   {'display_name': 'Liberia', 'id': 'lbr', 'name': 'lbr', 'title': 'Liberia'},
                                   {'display_name': 'Libya', 'id': 'lby', 'name': 'lby', 'title': 'Libya'},
                                   {'display_name': 'Saint Lucia', 'id': 'lca', 'name': 'lca', 'title': 'Saint Lucia'},
                                   {'display_name': 'Liechtenstein', 'id': 'lie', 'name': 'lie', 'title': 'Liechtenstein'},
                                   {'display_name': 'Sri Lanka', 'id': 'lka', 'name': 'lka', 'title': 'Sri Lanka'},
                                   {'display_name': 'Lesotho', 'id': 'lso', 'name': 'lso', 'title': 'Lesotho'},
                                   {'display_name': 'Lithuania', 'id': 'ltu', 'name': 'ltu', 'title': 'Lithuania'},
                                   {'display_name': 'Luxemburg', 'id': 'lux', 'name': 'lux', 'title': 'Luxemburg'},
                                   {'display_name': 'Latvia', 'id': 'lva', 'name': 'lva', 'title': 'Latvia'},
                                   {'display_name': 'Macau', 'id': 'mac', 'name': 'mac', 'title': 'Macau'},
                                   {'display_name': 'Morocco', 'id': 'mar', 'name': 'mar', 'title': 'Morocco'},
                                   {'display_name': 'Monaco', 'id': 'mco', 'name': 'mco', 'title': 'Monaco'},
                                   {'display_name': 'Moldova, Republic Of', 'id': 'mda', 'name': 'mda', 'title': 'Moldova, Republic Of'},
                                   {'display_name': 'Madagascar', 'id': 'mdg', 'name': 'mdg', 'title': 'Madagascar'},
                                   {'display_name': 'Maldives', 'id': 'mdv', 'name': 'mdv', 'title': 'Maldives'},
                                   {'display_name': 'Mexico', 'id': 'mex', 'name': 'mex', 'title': 'Mexico'},
                                   {'display_name': 'Marshall Islands', 'id': 'mhl', 'name': 'mhl', 'title': 'Marshall Islands'},
                                   {'display_name': 'Macedonia', 'id': 'mkd', 'name': 'mkd', 'title': 'Macedonia'},
                                   {'display_name': 'Mali', 'id': 'mli', 'name': 'mli', 'title': 'Mali'},
                                   {'display_name': 'Malta', 'id': 'mlt', 'name': 'mlt', 'title': 'Malta'},
                                   {'display_name': 'Myanmar', 'id': 'mmr', 'name': 'mmr', 'title': 'Myanmar'},
                                   {'display_name': 'Montenegro', 'id': 'mne', 'name': 'mne', 'title': 'Montenegro'},
                                   {'display_name': 'Mongolia', 'id': 'mng', 'name': 'mng', 'title': 'Mongolia'},
                                   {'display_name': 'Northern Mariana Islands', 'id': 'mnp', 'name': 'mnp', 'title': 'Northern Mariana Islands'},
                                   {'display_name': 'Mozambique', 'id': 'moz', 'name': 'moz', 'title': 'Mozambique'},
                                   {'display_name': 'Mauritania', 'id': 'mrt', 'name': 'mrt', 'title': 'Mauritania'},
                                   {'display_name': 'Montserrat', 'id': 'msr', 'name': 'msr', 'title': 'Montserrat'},
                                   {'display_name': 'Martinique', 'id': 'mtq', 'name': 'mtq', 'title': 'Martinique'},
                                   {'display_name': 'Mauritius', 'id': 'mus', 'name': 'mus', 'title': 'Mauritius'},
                                   {'display_name': 'Malawi', 'id': 'mwi', 'name': 'mwi', 'title': 'Malawi'},
                                   {'display_name': 'Malaysia', 'id': 'mys', 'name': 'mys', 'title': 'Malaysia'},
                                   {'display_name': 'Mayotte', 'id': 'myt', 'name': 'myt', 'title': 'Mayotte'},
                                   {'display_name': 'Namibia', 'id': 'nam', 'name': 'nam', 'title': 'Namibia'},
                                   {'display_name': 'New Caledonia', 'id': 'ncl', 'name': 'ncl', 'title': 'New Caledonia'},
                                   {'display_name': 'Nepal Earthquake', 'id': 'fba94eb6-a73b-407d-8acd-c16e61e4f0dc', 'name': 'nepal-earthquake', 'title': 'Nepal Earthquake'},
                                   {'display_name': 'Niger', 'id': 'ner', 'name': 'ner', 'title': 'Niger'},
                                   {'display_name': 'Norfolk Island', 'id': 'nfk', 'name': 'nfk', 'title': 'Norfolk Island'},
                                   {'display_name': 'Nigeria', 'id': 'nga', 'name': 'nga', 'title': 'Nigeria'},
                                   {'display_name': 'Nicaragua', 'id': 'nic', 'name': 'nic', 'title': 'Nicaragua'},
                                   {'display_name': 'Niue', 'id': 'niu', 'name': 'niu', 'title': 'Niue'},
                                   {'display_name': 'Netherlands', 'id': 'nld', 'name': 'nld', 'title': 'Netherlands'},
                                   {'display_name': 'Norway', 'id': 'nor', 'name': 'nor', 'title': 'Norway'},
                                   {'display_name': 'Nepal', 'id': 'npl', 'name': 'npl', 'title': 'Nepal'},
                                   {'display_name': 'Nauru', 'id': 'nru', 'name': 'nru', 'title': 'Nauru'},
                                   {'display_name': 'New Zealand', 'id': 'nzl', 'name': 'nzl', 'title': 'New Zealand'},
                                   {'display_name': 'Oman', 'id': 'omn', 'name': 'omn', 'title': 'Oman'},
                                   {'display_name': 'Pakistan', 'id': 'pak', 'name': 'pak', 'title': 'Pakistan'},
                                   {'display_name': 'Panama', 'id': 'pan', 'name': 'pan', 'title': 'Panama'},
                                   {'display_name': 'Pitcairn', 'id': 'pcn', 'name': 'pcn', 'title': 'Pitcairn'},
                                   {'display_name': 'Peru', 'id': 'per', 'name': 'per', 'title': 'Peru'},
                                   {'display_name': 'Philippines', 'id': 'phl', 'name': 'phl', 'title': 'Philippines'},
                                   {'display_name': 'Palau', 'id': 'plw', 'name': 'plw', 'title': 'Palau'},
                                   {'display_name': 'Papua New Guinea', 'id': 'png', 'name': 'png', 'title': 'Papua New Guinea'},
                                   {'display_name': 'Poland', 'id': 'pol', 'name': 'pol', 'title': 'Poland'},
                                   {'display_name': 'Puerto Rico', 'id': 'pri', 'name': 'pri', 'title': 'Puerto Rico'},
                                   {'display_name': "Democratic People's Republic of Korea", 'id': 'prk', 'name': 'prk', 'title': "Democratic People's Republic of Korea"},
                                   {'display_name': 'Portugal', 'id': 'prt', 'name': 'prt', 'title': 'Portugal'},
                                   {'display_name': 'Paraguay', 'id': 'pry', 'name': 'pry', 'title': 'Paraguay'},
                                   {'display_name': 'State of Palestine', 'id': 'pse', 'name': 'pse', 'title': 'State of Palestine'},
                                   {'display_name': 'French Polynesia', 'id': 'pyf', 'name': 'pyf', 'title': 'French Polynesia'},
                                   {'display_name': 'Qatar', 'id': 'qat', 'name': 'qat', 'title': 'Qatar'},
                                   {'display_name': 'Reunion', 'id': 'reu', 'name': 'reu', 'title': 'Reunion'},
                                   {'display_name': 'Romania', 'id': 'c92bd69f-54c0-4d02-ad30-d33fd1cd1393', 'name': 'rou', 'title': 'Romania'},
                                   {'display_name': 'Russia', 'id': 'rus', 'name': 'rus', 'title': 'Russia'},
                                   {'display_name': 'Rwanda', 'id': 'rwa', 'name': 'rwa', 'title': 'Rwanda'},
                                   {'display_name': 'Saudi Arabia', 'id': 'sau', 'name': 'sau', 'title': 'Saudi Arabia'},
                                   {'display_name': 'Sudan', 'id': 'sdn', 'name': 'sdn', 'title': 'Sudan'},
                                   {'display_name': 'Senegal', 'id': 'sen', 'name': 'sen', 'title': 'Senegal'},
                                   {'display_name': 'Singapore', 'id': 'sgp', 'name': 'sgp', 'title': 'Singapore'},
                                   {'display_name': 'South Georgia And South S.S.', 'id': 'sgs', 'name': 'sgs', 'title': 'South Georgia And South S.S.'},
                                   {'display_name': 'St. Helena', 'id': 'shn', 'name': 'shn', 'title': 'St. Helena'},
                                   {'display_name': 'Svalbard And Jan Mayen Islands', 'id': 'sjm', 'name': 'sjm', 'title': 'Svalbard And Jan Mayen Islands'},
                                   {'display_name': 'Solomon Islands', 'id': 'slb', 'name': 'slb', 'title': 'Solomon Islands'},
                                   {'display_name': 'Sierra Leone', 'id': 'sle', 'name': 'sle', 'title': 'Sierra Leone'},
                                   {'display_name': 'El Salvador', 'id': 'slv', 'name': 'slv', 'title': 'El Salvador'},
                                   {'display_name': 'San Marino', 'id': 'smr', 'name': 'smr', 'title': 'San Marino'},
                                   {'display_name': 'Somalia', 'id': 'som', 'name': 'som', 'title': 'Somalia'},
                                   {'display_name': 'St. Pierre And Miquelon', 'id': 'spm', 'name': 'spm', 'title': 'St. Pierre And Miquelon'},
                                   {'display_name': 'Serbia', 'id': 'srb', 'name': 'srb', 'title': 'Serbia'},
                                   {'display_name': 'South Sudan', 'id': 'ssd', 'name': 'ssd', 'title': 'South Sudan'},
                                   {'display_name': 'Sao Tome And Principe', 'id': 'stp', 'name': 'stp', 'title': 'Sao Tome And Principe'},
                                   {'display_name': 'Suriname', 'id': 'sur', 'name': 'sur', 'title': 'Suriname'},
                                   {'display_name': 'Slovakia (Slovak Republic)', 'id': 'svk', 'name': 'svk', 'title': 'Slovakia (Slovak Republic)'},
                                   {'display_name': 'Slovenia', 'id': 'svn', 'name': 'svn', 'title': 'Slovenia'},
                                   {'display_name': 'Sweden', 'id': 'swe', 'name': 'swe', 'title': 'Sweden'},
                                   {'display_name': 'Swaziland', 'id': 'swz', 'name': 'swz', 'title': 'Swaziland'},
                                   {'display_name': 'Seychelles', 'id': 'syc', 'name': 'syc', 'title': 'Seychelles'},
                                   {'display_name': 'Syrian Arab Republic', 'id': 'syr', 'name': 'syr', 'title': 'Syrian Arab Republic'},
                                   {'display_name': 'Turks And Caicos Islands', 'id': 'tca', 'name': 'tca', 'title': 'Turks And Caicos Islands'},
                                   {'display_name': 'Chad', 'id': 'tcd', 'name': 'tcd', 'title': 'Chad'},
                                   {'display_name': 'Togo', 'id': 'tgo', 'name': 'tgo', 'title': 'Togo'},
                                   {'display_name': 'Thailand', 'id': 'tha', 'name': 'tha', 'title': 'Thailand'},
                                   {'display_name': 'Tajikistan', 'id': 'tjk', 'name': 'tjk', 'title': 'Tajikistan'},
                                   {'display_name': 'Tokelau', 'id': 'tkl', 'name': 'tkl', 'title': 'Tokelau'},
                                   {'display_name': 'Turkmenistan', 'id': 'tkm', 'name': 'tkm', 'title': 'Turkmenistan'},
                                   {'display_name': 'East Timor', 'id': '86b999e4-6981-401e-b57d-e15ad5a9ec86', 'name': 'tls', 'title': 'East Timor'},
                                   {'display_name': 'Tonga', 'id': 'ton', 'name': 'ton', 'title': 'Tonga'},
                                   {'display_name': 'Trinidad And Tobago', 'id': 'tto', 'name': 'tto', 'title': 'Trinidad And Tobago'},
                                   {'display_name': 'Tunisia', 'id': 'tun', 'name': 'tun', 'title': 'Tunisia'},
                                   {'display_name': 'Turkey', 'id': 'tur', 'name': 'tur', 'title': 'Turkey'},
                                   {'display_name': 'Tuvalu', 'id': 'tuv', 'name': 'tuv', 'title': 'Tuvalu'},
                                   {'display_name': 'Taiwan, Province Of China', 'id': 'twn', 'name': 'twn', 'title': 'Taiwan, Province Of China'},
                                   {'display_name': 'United Republic of Tanzania', 'id': 'tza', 'name': 'tza', 'title': 'United Republic of Tanzania'},
                                   {'display_name': 'Uganda', 'id': 'uga', 'name': 'uga', 'title': 'Uganda'},
                                   {'display_name': 'Ukraine', 'id': 'ukr', 'name': 'ukr', 'title': 'Ukraine'},
                                   {'display_name': 'U.S. Minor Islands', 'id': 'umi', 'name': 'umi', 'title': 'U.S. Minor Islands'},
                                   {'display_name': 'Uruguay', 'id': 'ury', 'name': 'ury', 'title': 'Uruguay'},
                                   {'display_name': 'United States', 'id': 'usa', 'name': 'usa', 'title': 'United States'},
                                   {'display_name': 'Uzbekistan', 'id': 'uzb', 'name': 'uzb', 'title': 'Uzbekistan'},
                                   {'display_name': 'Holy See (Vatican City State)', 'id': 'vat', 'name': 'vat', 'title': 'Holy See (Vatican City State)'},
                                   {'display_name': 'Saint Vincent And The Grenadines', 'id': 'vct', 'name': 'vct', 'title': 'Saint Vincent And The Grenadines'},
                                   {'display_name': 'Venezuela', 'id': 'ven', 'name': 'ven', 'title': 'Venezuela'},
                                   {'display_name': 'Virgin Islands (British)', 'id': 'vgb', 'name': 'vgb', 'title': 'Virgin Islands (British)'},
                                   {'display_name': 'Virgin Islands (U.S.)', 'id': 'vir', 'name': 'vir', 'title': 'Virgin Islands (U.S.)'},
                                   {'display_name': 'Viet Nam', 'id': 'vnm', 'name': 'vnm', 'title': 'Viet Nam'},
                                   {'display_name': 'Vanuatu', 'id': 'vut', 'name': 'vut', 'title': 'Vanuatu'},
                                   {'display_name': 'Wallis And Futuna Islands', 'id': 'wlf', 'name': 'wlf', 'title': 'Wallis And Futuna Islands'},
                                   {'display_name': 'World', 'id': 'world', 'name': 'world', 'title': 'World'},
                                   {'display_name': 'Samoa', 'id': 'wsm', 'name': 'wsm', 'title': 'Samoa'},
                                   {'display_name': 'Yemen', 'id': 'yem', 'name': 'yem', 'title': 'Yemen'},
                                   {'display_name': 'South Africa', 'id': 'zaf', 'name': 'zaf', 'title': 'South Africa'},
                                   {'display_name': 'Zambia', 'id': 'zmb', 'name': 'zmb', 'title': 'Zambia'},
                                   {'display_name': 'Zimbabwe', 'id': 'zwe', 'name': 'zwe', 'title': 'Zimbabwe'}])


@pytest.fixture(scope='function')
def configuration(hdx_config_yaml, project_config_yaml):
    Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml,
                          project_config_yaml=project_config_yaml)


@pytest.fixture(scope='function')
def mocksmtp(monkeypatch):
    class MockSMTPBase(object):
        type = None

        def __init__(self, **kwargs):
            self.initargs = kwargs

        def login(self, username, password):
            self.username = username
            self.password = password

        def sendmail(self, sender, recipients, msg, **kwargs):
            self.sender = sender
            self.recipients = recipients
            self.msg = msg
            self.send_args = kwargs

        @staticmethod
        def quit():
            pass

    class MockSMTPSSL(MockSMTPBase):
        type = 'smtpssl'

    class MockLMTP(MockSMTPBase):
        type = 'lmtp'

    class MockSMTP(MockSMTPBase):
        type = 'smtp'
    monkeypatch.setattr(smtplib, 'SMTP_SSL', MockSMTPSSL)
    monkeypatch.setattr(smtplib, 'LMTP', MockLMTP)
    monkeypatch.setattr(smtplib, 'SMTP', MockSMTP)
